searchData={"items":[{"type":"behaviour","title":"LangSchema.Converter","doc":"Defines the behaviour and provides a macro for creating specific schema converters.\n\nThis module serves as a foundation for transforming an abstract schema definition\ninto a JSON schema compatible with a specific AI provider (like OpenAI, Gemini, etc.).\n\nIt defines the `convert/2` callback that all specific converter modules must implement\nand provides a `__using__` macro to inject basic conversion logic and allow for\ncustomization.","ref":"LangSchema.Converter.html"},{"type":"behaviour","title":"Usage - LangSchema.Converter","doc":"To create a new converter for a specific provider, you `use LangSchema.Converter`\nwithin your module definition and implement the required callbacks.\n\nSee `LangSchema.Converter.OpenAI` and `LangSchema.Converter.Gemini`.","ref":"LangSchema.Converter.html#module-usage"},{"type":"callback","title":"LangSchema.Converter.allowed_combinations/0","doc":"Returns the list of supported schema combination types.\n\nThis specifies which JSON Schema combinators are allowed during conversion.\nCommon values include `:any_of`, `:one_of`, and `:all_of`, corresponding to\n`anyOf`, `oneOf`, and `allOf` in the JSON Schema specification.\n\nYou can override this in a specific converter module if a provider supports only a subset.","ref":"LangSchema.Converter.html#c:allowed_combinations/0"},{"type":"type","title":"LangSchema.Converter.combination/0","doc":"","ref":"LangSchema.Converter.html#t:combination/0"},{"type":"callback","title":"LangSchema.Converter.convert/2","doc":"Converts a schema into a JSON schema.","ref":"LangSchema.Converter.html#c:convert/2"},{"type":"callback","title":"Options - LangSchema.Converter.convert/2","doc":"The supported options are:\n\n* `:ordered_properties` - If set to `true`, the properties of objects will be ordered\n  according to the order they are defined in the schema. Default is `false`.\n\n  When this option is enabled, the `properties` in the object schema should ideally be provided\n  as a keyword list to preserve the order, since maps in Elixir do not retain insertion order.\n  For providers like Gemini that support a separate `propertyOrdering` field, using a map may still work.\n  However, if you're targeting multiple AI providers, using a keyword list is recommended to ensure consistent ordering behavior.\n\n  Ordered properties may be serialized using `Jason.OrderedObject` to retain order in the\n  resulting JSON string. This assumes that `Jason` is used for final serialization; other encoders are not currently supported for ordered output.","ref":"LangSchema.Converter.html#c:convert/2-options"},{"type":"function","title":"LangSchema.Converter.convert/3","doc":"","ref":"LangSchema.Converter.html#convert/3"},{"type":"callback","title":"LangSchema.Converter.keywords/0","doc":"Returns a map of keyword-specific processors.\n\nEach key represents a JSON Schema keyword (e.g., `:pattern`, `:nullable`),\nand its value is a module responsible for processing that keyword during conversion.\n\nIf a keyword is not explicitly handled, the behavior defined in `LangSchema.Keyword.Default` is applied.\n\nSee `LangSchema.Keyword` for more details about keyword processing and extension.\n\nBy default, returns an empty map.","ref":"LangSchema.Converter.html#c:keywords/0"},{"type":"callback","title":"LangSchema.Converter.types/0","doc":"Returns a map of custom type converters.\n\nThis allows overriding or extending the default type conversion logic for each type.\nKeys must be atoms (e.g., `:string`, `:object`), and values must implement the corresponding conversion logic.\n\nSee `LangSchema.Type` modules for how to define custom type converters.\n\nBy default, returns an empty map.","ref":"LangSchema.Converter.html#c:types/0"},{"type":"callback","title":"LangSchema.Converter.wrap/2","doc":"Wraps the resulting JSON schema into a final structure required by the target provider.\n\nThis function is used when a provider requires the JSON schema to be embedded within\nan additional enclosing structure. For example, OpenAI's Chat Completion expects the schema\nto be placed under a `\"schema\"` field.\n\nThis function is also designed to be compatible with the `json_schema` input\nexpected by the [LangChain](https://hex.pm/packages/langchain) library.\n\nBy default, it returns the JSON schema as-is.\nYou can override this function in a specific converter module to modify the wrapping behavior.","ref":"LangSchema.Converter.html#c:wrap/2"},{"type":"module","title":"LangSchema.Converter.Gemini","doc":"","ref":"LangSchema.Converter.Gemini.html"},{"type":"module","title":"LangSchema.Converter.OpenAI","doc":"","ref":"LangSchema.Converter.OpenAI.html"},{"type":"module","title":"LangSchema.Converter.OpenAI.ChatCompletion","doc":"","ref":"LangSchema.Converter.OpenAI.ChatCompletion.html"},{"type":"function","title":"LangSchema.Converter.OpenAI.ChatCompletion.wrap/2","doc":"Wraps the given JSON schema into the structure expected by OpenAI's Chat API when using `response_format: \"json_schema\"`.\n\nThis implementation follows the specification described under:\n[OpenAI API Reference - Create Chat Completion](https://platform.openai.com/docs/api-reference/chat/create)\n(Request body → response_format → JSON schema → json_schema).\n\nIt sets the `name`, attaches the provided JSON schema under the `schema` field, and enforces `strict: true`.\nIf a `description` is provided in the options, it will be added as well.","ref":"LangSchema.Converter.OpenAI.ChatCompletion.html#wrap/2"},{"type":"module","title":"LangSchema.Converter.OpenAI.Function","doc":"","ref":"LangSchema.Converter.OpenAI.Function.html"},{"type":"behaviour","title":"LangSchema.Keyword","doc":"Defines the behaviour for modules that handle JSON Schema keywords.\n\nIf no specific keyword module exists, default behavior is provided by `LangSchema.Keyword.Default`.","ref":"LangSchema.Keyword.html"},{"type":"callback","title":"LangSchema.Keyword.convert/3","doc":"Converts and attaches a specific keyword to the JSON schema.\n\nThis function defines how a single keyword-value pair should be applied to the given JSON schema map.\n\nThe `keyword` argument is provided in snake_case format (e.g., `min_length`), and the implementer is responsible\nfor converting it to the appropriate camelCase JSON Schema field (e.g., `minLength`) if necessary.\n\nThis allows fine-grained control over the transformation of each keyword during schema generation.","ref":"LangSchema.Keyword.html#c:convert/3"},{"type":"module","title":"LangSchema.Keyword.Default","doc":"","ref":"LangSchema.Keyword.Default.html"},{"type":"module","title":"LangSchema.Keyword.OpenAI.Nullable","doc":"","ref":"LangSchema.Keyword.OpenAI.Nullable.html"},{"type":"module","title":"LangSchema.Keyword.Pattern","doc":"","ref":"LangSchema.Keyword.Pattern.html"},{"type":"behaviour","title":"LangSchema.Type","doc":"Defines the behaviour and common helpers for handling JSON Schema type conversion.\n\nEach module implementing `LangSchema.Type` specifies how a particular JSON Schema `type`\n(such as `boolean`, `integer`, `number`, `string`, `array`, or `object`) should be converted\ninto its corresponding JSON Schema representation.\n\nIt also provides shared utilities for processing schema keywords during conversion.","ref":"LangSchema.Type.html"},{"type":"function","title":"LangSchema.Type.array_keywords/0","doc":"","ref":"LangSchema.Type.html#array_keywords/0"},{"type":"function","title":"LangSchema.Type.common_keywords/0","doc":"","ref":"LangSchema.Type.html#common_keywords/0"},{"type":"callback","title":"LangSchema.Type.convert/3","doc":"Defines how the core part of the given type should be converted into its corresponding JSON Schema representation.\n\nThis focuses on converting the essential structure specific to the type — for example,\nhandling `properties` for `object`, or `items` for `array`.\n\nComplex types like `object` and `array` will typically implement meaningful logic here.","ref":"LangSchema.Type.html#c:convert/3"},{"type":"callback","title":"LangSchema.Type.convert_keywords/3","doc":"Converts and attaches additional keywords to the JSON schema for this type.\n\nThis function applies the allowed keywords (as defined by `keywords/0`) onto the base JSON schema\naccording to the values provided in the original schema definition.\n\nBy default, the `pattern` keyword is predefined with its specific converter.\nIf a keyword is not explicitly associated with a converter, the behavior defined by `LangSchema.Keyword.Default` is applied.","ref":"LangSchema.Type.html#c:convert_keywords/3"},{"type":"function","title":"LangSchema.Type.convert_keywords/4","doc":"","ref":"LangSchema.Type.html#convert_keywords/4"},{"type":"callback","title":"LangSchema.Type.keywords/0","doc":"Returns the list of allowed keywords that can be additionally set for this type.\n\nIn JSON Schema, various keywords (such as `description`, `minimum`, `pattern`, etc.)\ncan be used to further constrain or describe a field beyond its basic type.\n\nThis callback defines which keywords are considered relevant for the type\nand will be processed during schema conversion.\n\nWithin this Elixir codebase, keyword names follow Elixir's snake_case convention\n(e.g., `min_length`, `max_items`), even though in JSON Schema they are typically camelCase\n(e.g., `minLength`, `maxItems`).","ref":"LangSchema.Type.html#c:keywords/0"},{"type":"function","title":"LangSchema.Type.numeric_keywords/0","doc":"","ref":"LangSchema.Type.html#numeric_keywords/0"},{"type":"function","title":"LangSchema.Type.object_keywords/0","doc":"","ref":"LangSchema.Type.html#object_keywords/0"},{"type":"function","title":"LangSchema.Type.string_keywords/0","doc":"","ref":"LangSchema.Type.html#string_keywords/0"},{"type":"module","title":"LangSchema.Type.Array","doc":"","ref":"LangSchema.Type.Array.html"},{"type":"module","title":"LangSchema.Type.Boolean","doc":"","ref":"LangSchema.Type.Boolean.html"},{"type":"module","title":"LangSchema.Type.Gemini.Array","doc":"","ref":"LangSchema.Type.Gemini.Array.html"},{"type":"module","title":"LangSchema.Type.Gemini.Integer","doc":"","ref":"LangSchema.Type.Gemini.Integer.html"},{"type":"module","title":"LangSchema.Type.Gemini.Number","doc":"","ref":"LangSchema.Type.Gemini.Number.html"},{"type":"module","title":"LangSchema.Type.Gemini.Object","doc":"","ref":"LangSchema.Type.Gemini.Object.html"},{"type":"module","title":"LangSchema.Type.Gemini.String","doc":"","ref":"LangSchema.Type.Gemini.String.html"},{"type":"module","title":"LangSchema.Type.Integer","doc":"","ref":"LangSchema.Type.Integer.html"},{"type":"module","title":"LangSchema.Type.Number","doc":"","ref":"LangSchema.Type.Number.html"},{"type":"module","title":"LangSchema.Type.Object","doc":"","ref":"LangSchema.Type.Object.html"},{"type":"module","title":"LangSchema.Type.OpenAI.Array","doc":"","ref":"LangSchema.Type.OpenAI.Array.html"},{"type":"module","title":"LangSchema.Type.OpenAI.Integer","doc":"","ref":"LangSchema.Type.OpenAI.Integer.html"},{"type":"module","title":"LangSchema.Type.OpenAI.Number","doc":"","ref":"LangSchema.Type.OpenAI.Number.html"},{"type":"module","title":"LangSchema.Type.OpenAI.Object","doc":"","ref":"LangSchema.Type.OpenAI.Object.html"},{"type":"module","title":"LangSchema.Type.OpenAI.String","doc":"","ref":"LangSchema.Type.OpenAI.String.html"},{"type":"module","title":"LangSchema.Type.String","doc":"","ref":"LangSchema.Type.String.html"},{"type":"module","title":"LangSchema.Util.Nillable","doc":"","ref":"LangSchema.Util.Nillable.html"},{"type":"function","title":"LangSchema.Util.Nillable.run_if/3","doc":"","ref":"LangSchema.Util.Nillable.html#run_if/3"},{"type":"extras","title":"Overview","doc":"# LangSchema\n\nConverts an abstract schema into JSON schemas required by various AI providers, minimizing code changes when switching providers.","ref":"readme.html"},{"type":"extras","title":"Design Goals - Overview","doc":"LangSchema aims to define an abstract schema that can be commonly applied across different AI providers, allowing you to switch providers without modifying your original schema. While provider-specific features are not entirely excluded, supporting every custom feature is not the primary goal. Instead, LangSchema prioritizes raising errors when a schema might implicitly behave incorrectly due to differences between providers, making such issues explicit and easier to fix.","ref":"readme.html#design-goals"},{"type":"extras","title":"Concepts - Overview","doc":"- **Abstract Schema**: A subset of JSON Schema that is primarily used to define structured outputs or function parameters for AI providers. It aims to maintain an independent expression without being tied to any specific provider. For an example, see [schema.ex](https://github.com/nallwhy/lang_schema/blob/main/test/support/schema.ex).\n- **JSON Schema**: The concrete output generated by converting the abstract schema to match the specific requirements of each AI provider.","ref":"readme.html#concepts"},{"type":"extras","title":"Supported AI providers - Overview","doc":"- OpenAI\n  - [LangSchema.Converter.OpenAI.ChatCompletion](./lib/lang_schema/converter/openai/chat_completion.ex)\n  - [LangSchema.Converter.OpenAI.Function](./lib/lang_schema/converter/openai/function.ex)\n- Gemini\n  - [LangSchema.Converter.Gemini](./lib/lang_schema/converter/gemini.ex)","ref":"readme.html#supported-ai-providers"},{"type":"extras","title":"Writing a Custom Converter - Overview","doc":"You can create your own converter for a new AI provider by implementing the `LangSchema.Converter` behaviour.\n\nHere's a simple example based on the OpenAI converter:\n\n```elixir\ndefmodule MyApp.LangSchema.Converter.MyProvider do\n  use LangSchema.Converter\n\n  @impl LangSchema.Converter\n  def wrap(json_schema, opts) do\n    name = Keyword.get(opts, :name, \"response\")\n\n    %{\n      \"name\" => name,\n      \"schema\" => json_schema\n    }\n  end\n\n  @impl LangSchema.Converter\n  def types() do\n    # Add type converters\n    %{\n      integer: MyApp.LangSchema.Type.MyProvider.Integer,\n      number: MyApp.LangSchema.Type.MyProvider.Number,\n      string: MyApp.LangSchema.Type.MyProvider.String,\n      array: MyApp.LangSchema.Type.MyProvider.Array,\n      object: MyApp.LangSchema.Type.MyProvider.Object\n    }\n  end\n\n  @impl LangSchema.Converter\n  def allowed_combinations() do\n    # List only the combinations your provider supports, e.g., :any_of, :one_of, :all_of\n    [:any_of] \n  end\n\n  @impl LangSchema.Converter\n  def keywords() do\n    %{\n      # Add keyword converters\n      nullable: MyApp.LangSchema.Keyword.MyProvider.Nullable\n    }\n  end\nend\n```","ref":"readme.html#writing-a-custom-converter"},{"type":"extras","title":"Installation - Overview","doc":"The package can be installed by adding `lang_schema` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:lang_schema, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Relationship with Elixir LangChain - Overview","doc":"Elixir [LangChain](https://hex.pm/packages/langchain) library provides support for function calling by accepting a `json_schema`.\nLangSchema is designed so that the output of each converter’s `wrap/2` function can be directly used as a json_schema in LangChain without additional modifications.","ref":"readme.html#relationship-with-elixir-langchain"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}